from math import sqrt
from libs.ustr import ustr
import hashlib
try:
    from PyQt5.QtGui import *
    from PyQt5.QtCore import *
    from PyQt5.QtWidgets import *
except ImportError:
    from PyQt4.QtGui import *
    from PyQt4.QtCore import *


def newIcon(icon):
    return QIcon(':/' + icon)


def newButton(text, icon=None, slot=None):
    b = QPushButton(text)
    if icon is not None:
        b.setIcon(newIcon(icon))
    if slot is not None:
        b.clicked.connect(slot)
    return b


def makeShortcut(parent, binding, action):
    shortcut = QShortcut(QKeySequence(binding), parent)
    shortcut.activated.connect(action)
    return shortcut

def do_nothing(_):
    pass

def makeShortcuts(parent, bindings, action, collect=do_nothing):
    for binding in bindings:
        collect(makeShortcut(parent, binding, action))

def newAction(parent, text, slot=None, key_sequence=None, icon=None, tip=None, checkable=False, enabled=True, forParent=False):
    """Create a new action and assign callbacks, shortcuts, etc."""
    a = QAction(text, parent)
    if icon is not None:
        a.setIcon(newIcon(icon))
    if key_sequence is not None:
        if not forParent:
            if isinstance(key_sequence, (list, tuple)):
                a.setShortcuts(key_sequence)
            else:
                a.setShortcut(key_sequence)
        elif slot is not None:
            if isinstance(key_sequence, (list, tuple)):
                makeShortcuts(parent, key_sequence, slot)
            else:
                makeShortcut(parent, key_sequence, slot)
    if tip is not None:
        a.setToolTip(tip)
        a.setStatusTip(tip)
    if slot is not None and not forParent:
        a.triggered.connect(slot)
    if checkable:
        a.setCheckable(True)
    a.setEnabled(enabled)
    return a


def addActions(widget, actions):
    for action in actions:
        if action is None:
            widget.addSeparator()
        elif isinstance(action, QMenu):
            widget.addMenu(action)
        else:
            widget.addAction(action)


def labelValidator():
    return QRegExpValidator(QRegExp(r'^[^ \t].+'), None)


class struct(object):

    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)


def distance(p):
    return sqrt(p.x() * p.x() + p.y() * p.y())


def fmtShortcut(text):
    mod, key = text.split('+', 1)
    return '<b>%s</b>+<b>%s</b>' % (mod, key)


def generateColorByText(text):
    s = ustr(text)
    hashCode = int(hashlib.sha256(s.encode('utf-8')).hexdigest(), 16)
    r = int((hashCode / 255) % 255)
    g = int((hashCode / 65025)  % 255)
    b = int((hashCode / 16581375)  % 255)
    return QColor(r, g, b, 100)
